#pragma once

#include <iostream>
#include <stdexcept>

namespace rational {

// Represents a rational number, that is, a fraction.
struct Rational
{
    long num, den;
    // Invariants:
    //   gcd(num, den) == 1
    //   den > 0
};

Rational make_rational();
Rational make_rational(long value);
Rational make_rational(long num, long den);

bool operator==(const Rational&, const Rational&);
bool operator!=(const Rational&, const Rational&);
bool operator<(const Rational&, const Rational&);
bool operator<=(const Rational&, const Rational&);
bool operator>(const Rational&, const Rational&);
bool operator>=(const Rational&, const Rational&);

std::ostream& operator<<(std::ostream&, const Rational&);
double to_double(const Rational&);

Rational operator-(const Rational&);
Rational reciprocal(const Rational&);

Rational operator+(const Rational&, const Rational&);
Rational operator*(const Rational&, const Rational&);
Rational operator-(const Rational&, const Rational&);
Rational operator/(const Rational&, const Rational&);

Rational& operator+=(Rational&, const Rational&);
Rational& operator-=(Rational&, const Rational&);
Rational& operator*=(Rational&, const Rational&);
Rational& operator/=(Rational&, const Rational&);

Rational& operator++(Rational&);
Rational& operator--(Rational&);
Rational operator++(Rational&, int);
Rational operator--(Rational&, int);

}
